"""hooks triggered on email entities creation:

* look for state change instruction (XXX security)
* set email content as a comment on an entity when comments are supported and
  linking information are found

:organization: Logilab
:copyright: 2003-2008 LOGILAB S.A. (Paris, FRANCE), all rights reserved.
:contact: http://www.logilab.fr/ -- mailto:contact@logilab.fr
"""
__docformat__ = "restructuredtext en"

import re

from logilab.mtconverter import TransformError

from cubicweb import UnknownEid
from cubicweb.common.schema import FormatConstraint
from cubicweb.server.pool import PreCommitOperation
from cubicweb.server.hooksmanager import Hook
from cubicweb.sobjects.notification import parse_message_id


def fix_ownership(session, eid, email):
    sender = email.senderaddr.email_of
    if (sender and sender.e_schema == 'EUser' and
        sender.eid != session.actual_session().user.eid):
        # match a user which is not the session's user, set owned_by / created_by
        session.unsafe_execute('SET X owned_by U WHERE X eid %(x)s, U eid %(u)s',
                               {'x': eid, 'u': sender.eid}, 'x')
        session.unsafe_execute('SET X created_by U WHERE X eid %(x)s, U eid %(u)s',
                               {'x': eid, 'u': sender.eid}, 'x')


class ExtractEmailInformation(PreCommitOperation):
    """generate a comment on the original entity if supported"""
    
    def precommit_event(self):
        email = self.email
        # should create a Comment ?
        info = self.info
        origeid = int(info['eid'])
        try:
            origetype = self.session.describe(origeid)[0]
        except UnknownEid:
            self.error('email %s is referencing an unknown eid %s',
                       email.messageid, origeid)
            return
        if origetype in self.schema.rschema('comments').objects('Comment'):
            try:
                part = email.parts_in_order(prefered_mime_type='text/plain')[0]
            except IndexError:
                pass
            else:
                try:
                    self.insert_comment(origeid, part)
                except:
                    self.exception('while generating comment on %s from email %s',
                                   origeid, email)
                    
    def insert_comment(self, eid, emailpart):
        com = self.session.unsafe_execute(
            'INSERT Comment C: C content %(content)s, '
            'C content_format %(format)s, C comments X, C generated_by E '
            'WHERE X eid %(x)s, E eid %(e)s',
            {'x': eid, 'e': self.email.eid, 'format': u'text/plain',
             'content': emailpart.actual_content()}, 'x')
        fix_ownership(self.session, com[0][0], self.email)


class ExtractEmailChangeState(PreCommitOperation):
    """check if there are some change state instruction in the mail content"""
    
    instr_rgx = re.compile(':(\w+):\s*#?(\d+)', re.U)
    
    def precommit_event(self):
        # should pass some transitions ?
        text = self.email.subject
        for part in self.email.parts_in_order(prefered_mime_type='text/plain'):
            try:
                text += ' ' + part.actual_content()
            except TransformError:
                continue
        for trname, eid in self.instr_rgx.findall(text):
            eid = int(eid)
            try:
                entity = self.session.entity(eid)
            except UnknownEid:
                self.error("can't get entity with eid %s", eid)
                continue
            if not hasattr(entity, 'in_state'):
                self.error('bad change state instruction for eid %s', eid)
                continue
            transition = entity.can_pass_transition(trname)
            if transition:
                try:
                    self.change_state(entity, transition)
                except:
                    self.exception('while changing state off %s from email %s',
                                   entity, self.email)
            else:
                self.error("can't pass transition %s on entity %s",
                           trname, entity)
                                
    def change_state(self, entity, transition):
        entity.change_state(transition.destination_state[0].eid)
        trinfo = entity.latest_trinfo()
        self.session.execute('SET X generated_by S WHERE X eid %(x)s, S eid %(s)s',
                             {'x': trinfo.eid, 's': self.email.eid}, 'x')
        fix_ownership(self.session, trinfo.eid, self.email)

    
class AddEmailHook(Hook):
    """an email has been added, check if associated content should be created
    """
    events = ('after_add_entity',)
    accepts = ('Email',)
    
    def call(self, session, entity):
        if 'comments' in session.repo.schema:
            for msgid in entity.references():
                info = parse_message_id(msgid, self.config.appid)
                self.info('extracted information from message id %s: %s',
                          msgid, info)
                if info:
                    ExtractEmailInformation(session, email=entity, info=info)
                    break
        ExtractEmailChangeState(session, email=entity)
