"""Specific views for email related entities

:organization: Logilab
:copyright: 2003-2008 LOGILAB S.A. (Paris, FRANCE), all rights reserved.
:contact: http://www.logilab.fr/ -- mailto:contact@logilab.fr
"""

from logilab.mtconverter import html_escape

from cubicweb.common.uilib import soup2xhtml
from cubicweb.common.mixins import TreeViewMixIn
from cubicweb.web.views import baseviews


class EmailPrimaryView(baseviews.PrimaryView):
    accepts = ('Email',)
    skip_rels = ('sender', 'recipients', 'cc', 'parts', 'owned_by')

    def render_entity_attributes(self, entity, siderelations):
        self.field('from', entity.sender[0].view('oneline'))
        self.field('to', ', '.join(ea.view('oneline') for ea in entity.recipients))
        if entity.cc:
            self.field('cc', ', '.join(ea.view('oneline') for ea in entity.cc))
        self.field('subject', entity.subject)
        self.field('date', self.format_date(entity.date, time=True))
        for part in entity.parts_in_order():
            content, mime = part.content, part.content_format
            if mime == 'text/html':
                content = soup2xhtml(content, self.req.encoding)
            elif mime != 'text/xhtml':
                content = html_escape(content)
                if mime == 'text/plain':
                    content = content.replace('\n', '<br/>').replace(' ', '&nbsp;')
            # XXX some headers to skip if html ?
            self.w(content)
            self.w(u'<br class="partseparator"/>')
            
#    def render_side_related(self, entity, siderelations):
#        pass

    def content_title(self, entity):
        return entity.dc_title()
    
    
class EmailHeadersView(baseviews.EntityView):
    """display email's headers"""
    id = 'headers'
    title = _('headers')
    accepts = ('Email',)
    templatable = False
    content_type = 'text/plain'
    
    def cell_call(self, row, col):
        entity = self.entity(row, col)
        self.w(entity.headers)

    
class EmailOneLineView(baseviews.PrimaryView):
    """short view usable in the context of the email sender/recipient (in which
    case the caller should specify its context eid) or outside any context
    """
    id = 'oneline'
    title = _('oneline')
    accepts = ('Email',)
    
    def cell_call(self, row, col, contexteid=None):
        entity = self.entity(row, col)
        sender = entity.sender[0]
        self.w(u'<div class="email">')
        self.w(u'<i>%s&nbsp;%s</i> '
               % (self.req._('email_date'), self.format_date(entity.date, time=True)))
        if contexteid != sender.eid:
            self.w(u'<b>%s</b>&nbsp;%s '
                   % (self.req._('email_from'), sender.view('oneline')))
        if contexteid not in (r.eid for r in entity.recipients):
            recipients = ', '.join(r.view('oneline') for r in entity.recipients)
            self.w(u'<b>%s</b>&nbsp;%s'
                   % (self.req._('email_to'), recipients))
        self.w(u'<br/>\n<a href="%s">%s</a>'
               % (html_escape(entity.absolute_url()), html_escape(entity.subject)))
        self.w(u'</div>')

class EmailOutOfContextView(EmailOneLineView):
    """short view outside the context of the email"""
    id = 'outofcontext'
    title = _('out of context') 

class EmailInContextView(EmailOneLineView):
    """short view inside the context of the email"""
    id = 'incontext'


class EmailPartOutOfContextView(baseviews.OutOfContextView):
    """out of context an email part is redirecting to related email view"""
    accepts = ('EmailPart',)
    def cell_call(self, row, col):
        entity = self.entity(row, col)
        entity.reverse_parts[0].view('outofcontext', w=self.w)
        
        
class EmailThreadView(TreeViewMixIn, baseviews.ListView):
    """display email's headers"""
    accepts = ('Email',)
    title = _('thread view')
    item_vid = 'outofcontext'

            
class EmailThreadPrimaryView(baseviews.PrimaryView):
    accepts = ('EmailThread',)
    
    def cell_call(self, row, col):
        entity = self.complete_entity(row, col)
        self.w(u'<h1>%s</h1>' % html_escape(entity.title))
        # get top level emails in this thread (ie message which are not a reply
        # of a message in this thread)
        #
        # Warn: adding Y in_thread E changes the meaning of the query since it joins
        # with messages which are not a direct reply (eg if A reply_to B, B reply_to C
        # B is also retreived since it's not a reply of C
        #
        # XXX  union with
        #   DISTINCT Any X,D ORDERBY D WHERE X date D, X in_thread E, X reply_to Y,
        #   NOT Y in_thread E, E eid %(x)s'
        # to get message which are a reply of a message in another thread ?
        # we may get duplicates in this case
        rset =  self.req.execute('DISTINCT Any X,D ORDERBY D '
                                 'WHERE X date D, X in_thread E, '
                                 'NOT X reply_to Y, E eid %(x)s',
                                 {'x': entity.eid}, 'x')
        if rset:
            self.w(u'<ul>')
            self.w(u'\n'.join(email.view('tree') for email in rset.entities()))
            self.w(u'</ul>')
            
        
